
#ifndef TEST_H
#define TEST_H


// Test is a base class for all tests.  It provides a command interface for
// running tests (run) as well as a data member for recording the name of 
// the test.
//
// Tests are constructed using the TEST macro.  TEST creates a subclass of
// Test and static instance of that subclass.  If you look at the constructor
// for the Test class, you'll notice that it registers the created object 
// with a global TestRegistry.  These features combine to make test creation
// particularly easy.


#include <vector>
#include <string>

class TestResult;


class Test
{
public:
	Test (const std::string& testName);

	virtual void	run (TestResult& result);
	virtual void	runTest (TestResult& result) = 0;

protected:
	std::string		name;

};



#define TEST(name,classUnderTest)\
	class classUnderTest##name##Test : public Test\
	{ \
		public: \
			classUnderTest##name##Test () : Test (#name "Test") {} \
			void runTest (TestResult& result_); \
	} classUnderTest##name##Instance; \
	void classUnderTest##name##Test::runTest (TestResult& result_) \



// Here is a collection of testing macros that can be used in the 
// bodies of tests.  CHECK tests a boolean expression and records
// a failure if the expression evaluates to false.  CHECK_LONGS_EQUAL
// and CHECK_DOUBLES_EQUAL compare longs and doubles respectively.
//
// To make this an industrial strength test harness, you should
// add equals macros for various low level types as you develop them.
// If, for instance, you have a daterange class, the ability to compare
// them directly and print out their values in the test output is 
// invaluable.




#define CHECK(condition) \
	if (!(condition)) \
		result_.addFailure (Failure (#condition, name, __FILE__, __LINE__));

#define CHECK_LONGS_EQUAL(expected,actual)\
{\
	long _expected = (expected);\
	long _actual = (actual);\
	if (_expected != _actual) {\
		char message [80];\
		sprintf (message, "expected %ld but was: %ld", _expected, _actual);\
		result_.addFailure (Failure (message, name, __FILE__, __LINE__));\
	}\
}


#define CHECK_DOUBLES_EQUAL(expected,actual)\
{\
	double _expected = (expected);\
	double _actual = (actual);\
	if (fabs ((expected)-(actual)) > 0.001) {\
		char message [80];\
		sprintf (message, "expected %lf but was: %lf", (expected), (actual));\
		result_.addFailure (Failure (message, name, __FILE__, __LINE__));\
	}\
}


#define CHECK_ERRNO(returnval, expected)\
	do {\
		CHECK_LONGS_EQUAL(-1, (returnval));\
		CHECK_LONGS_EQUAL((expected), errno);\
	} while (0)


#define CHECK_TMS_EQUAL(e, a)\
	do {\
		char msg[1024];\
		bzero(msg, sizeof msg);\
		if ((e).tm_sec != (a).tm_sec\
		    || (e).tm_min != (a).tm_min\
		    || (e).tm_hour != (a).tm_hour\
		    || (e).tm_mday != (a).tm_mday\
		    || (e).tm_mon != (a).tm_mon\
		    || (e).tm_year != (a).tm_year\
		    || (e).tm_wday != (a).tm_wday\
		    || (e).tm_yday != (a).tm_yday\
		    || (e).tm_isdst != (a).tm_isdst\
		    || (e).tm_gmtoff != (a).tm_gmtoff\
		    || strncmp((e).tm_zone, (a).tm_zone, 3) != 0) {\
			snprintf(msg, sizeof msg, "struct tm are not equal!\n"\
			    "%-10s %-8s %-8s\n"\
			    "%-10s %-8d %-8d\n"\
			    "%-10s %-8d %-8d\n"\
			    "%-10s %-8d %-8d\n"\
			    "%-10s %-8d %-8d\n"\
			    "%-10s %8-d %-8d\n"\
			    "%-10s %-8d %-8d\n"\
			    "%-10s %-8d %-8d\n"\
			    "%-10s %-8d %-8d\n"\
			    "%-10s %8-d %-8d\n"\
			    "%-10s %-8d %-8d\n"\
			    "%-10s %-8s %-8s\n",\
			    "Field", "Expected", "Actual",\
			    "tm_sec", (e).tm_sec, (a).tm_sec,\
			    "tm_min", (e).tm_min, (a).tm_min,\
			    "tm_hour", (e).tm_hour, (a).tm_hour,\
			    "tm_mday", (e).tm_mday, (a).tm_mday,\
			    "tm_mon", (e).tm_mon, (a).tm_mon,\
			    "tm_year", (e).tm_year, (a).tm_year,\
			    "tm_wday", (e).tm_wday, (a).tm_wday,\
			    "tm_yday", (e).tm_yday, (a).tm_yday,\
			    "tm_isdst", (e).tm_isdst, (a).tm_isdst,\
			    "tm_gmtoff", (e).tm_gmtoff, (a).tm_gmtoff,\
			    "tm_zone", (e).tm_zone, (a).tm_zone);\
			result_.addFailure (Failure (msg, name, __FILE__, __LINE__));\
		}\
	} while (0)

#endif



